;  delay.inc - helper macro to delay for a specified number
;  of CPU cylces (min. 2)
;
;  Copyright (c) 2010 by Matthias Reichl <hias@horus.com>
;
;  This program is free software; you can redistribute it and/or modify
;  it under the terms of the GNU General Public License as published by
;  the Free Software Foundation; either version 2 of the License, or
;  (at your option) any later version.
;
;  This program is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;
;  You should have received a copy of the GNU General Public License
;  along with this program; if not, write to the Free Software
;  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

; note: you need at least ATasm 1.06 to assemble this file.
; ATasm 1.05 and before don't support ".rept expression"

; delay for given number of cycles
;
; this macro generates code using a combination of
;   DELAYX : x*5+16+2 cycles
;   LDX 0  :   3      cycles
;   NOP    : x*2      cycles

.MACRO DELAY
    .IF %0 <> 1
        .ERROR "macro DELAY needs a parameter"
    .ENDIF
    .IF %1 <= 1
        .ERROR "DELAY parameter has to be at least 2"
    .ENDIF
    
    DREMAIN .= %1
    
    ; special case: 18 cycles - just use DELAYX
    .IF DREMAIN = 18
        LDX #1		; 2
        JSR DELAYX	; 16
        DREMAIN .= 0	; we are finished
    .ENDIF
    
    ; for 19 cycles we have to use NOP/LDX
    ; use DELAYX for >= 20 cycles
    .IF DREMAIN >= 20
	; calculate number of DELAYX loops
        DLOOPS .= [DREMAIN-18]/5
	; special case: only one remaining cycle
        .IF DREMAIN - DLOOPS*5 - 18 = 1
            ; 6502 can't wait for one cycle, so decrease
            ; loop counter by one (remaining 6 cycles will
            ; be delayed using NOPs)
    	    DLOOPS .= DLOOPS - 1
        .ENDIF
        LDX #DLOOPS+1	; 2
        ; DLOOPS+1 because DELAYX delays for (X-1)*5+16
        JSR DELAYX	; 5*(X-1)+16
        ; adjust for already delayed time
        DREMAIN .= DREMAIN - DLOOPS*5 - 18
    .ENDIF
    
    ; if remaining time is odd, delay for 3 cycles to get
    ; to an even number of remaining cycles
    ; note: DREMAIN = 1 cannot happen, the code before
    ; already takes care of this.
    .IF DREMAIN & 1
        LDX 0
        DREMAIN .= DREMAIN - 3
    .ENDIF
    
    ; now the remaining time is always an even number (or 0).
    ; spend this time doing NOPs
    .IF DREMAIN > 0
        .REPT DREMAIN/2
    	NOP
        .ENDR
    .ENDIF
.ENDM

